<?php

namespace Drupal\payment_authnet;

/**
 * Provides helper functions for additional fields.
 */
trait AdditionalFieldsTrait {

  /**
   * Returns the list of additional fields to be sent to Authorize.net.
   *
   * @return array
   *   The list of available / supported fields grouped by categories.
   */
  protected function getAvailableFields() {
    return [
      'Billing Information' => [
        'first_name' => 'textfield',
        'last_name' => 'textfield',
      ],
    ];
  }

  /**
   * Converts human readable string into machine name.
   *
   * @param string $human_readable
   *   Human readable string.
   *
   * @return string
   *   Machine name (allowed characters - lowercase letters, numbers and '_').
   */
  protected function humanToMachine($human_readable) {
    return preg_replace('@[^a-z0-9_]+@', '_', strtolower($human_readable));
  }

  /**
   * Builds section wrapper for additional fields.
   *
   * @param string $section_human_name
   *   Section title.
   *
   * @return array
   *   Form element wrapper array ('details' type).
   */
  protected function buildSectionWrapper($section_human_name) {
    $section_name = $this->humanToMachine($section_human_name);
    $result[$section_name] = [
      '#type' => 'details',
      '#title' => t($section_human_name),
      '#tree' => TRUE,
    ];
    return $result;
  }

  /**
   * Creates fields in addition to credit card number, date and cvv.
   *
   * @return array
   *   An array of additional fields to adjust Configuration Form.
   */
  protected function addAdditionalFields() {
    $result = [];
    $field_mapping = \Drupal::config('payment_authnet.settings')->get('field_mappings');
    foreach ($this->getAvailableFields() as $section_human_name => $children) {
      $section_name = $this->humanToMachine($section_human_name);
      $result += $this->buildSectionWrapper($section_human_name);
      foreach ($children as $field_name => $type) {
        if (!empty($field_mapping[$section_name][$field_name]['title'])) {
          $result[$section_name][$field_name] = [
            '#type' => $type,
            '#title' => $field_mapping[$section_name][$field_name]['title'],
            '#required' => $field_mapping[$section_name][$field_name]['required'],
            '#default_value' => $this->token->replace($field_mapping[$section_name][$field_name]['default_value']),
          ];
          if ($field_mapping[$section_name][$field_name]['required']) {
            $result[$section_name]['#open'] = TRUE;
            $result[$section_name]['#required'] = TRUE;
          }
        }
      }
    }
    return $result;
  }

  /**
   * Additional fields default values for defaultConfiguration().
   *
   * @return array
   *   An array with additional fields default values.
   */
  protected function additionalFieldsDefaultConfiguration() {
    $result = [];
    $field_mapping = \Drupal::config('payment_authnet.settings')->get('field_mappings');
    foreach ($this->getAvailableFields() as $section_human_name => $children) {
      $section_name = $this->humanToMachine($section_human_name);
      foreach (array_keys($children) as $field_name) {
        if (!empty($field_mapping[$section_name][$field_name]['title'])) {
          $result[$section_name][$field_name] = $this->token->replace($field_mapping[$section_name][$field_name]['default_value']);
        }
      }
    }
    return $result;
  }

}
